//
// halffull.2.2.c
//
// Control a charlieplexed LED matrix through serial communication
//
// Shahar Ronen 9/12/11
//
// Open issues: 
// Messages are printed twice (block interrupts while handling one?),
// more scalable way to pass information on fluid height.
//
// Changes from 2.1: recipeTypes are hardcoded in the controller
// Changes from 2.0: decoding works partially, only for one column...
// Changes from 1.2: added decoding of lighting info bit-by-bit for better scaling
// Changes from 1.1: exported functions to .h files, added interrupt support
// Changes from 1.0: added serial communication, added array that maps the pins
//
// Based on:
// hello.array.44.c: Charlieplex LED array hello-world
// hello.ftdi.44.echo.interrupt.c: serial comm through interrupts
//
// Neil Gershenfeld
// 11/13/10
//
// (c) Massachusetts Institute of Technology 2010
// Permission granted for experimental and personal use;
// license for commercial sale available from MIT.
//


#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h> 

#include "led_display.h"
#include "led_serial.h"

// For PCINT7..0, use: PCIE0, PCINT0_vect, PCMSK0
// For PCINT11..8, use: PCIE1, PCINT1_vect, PCMSK1
#define serial_interrupt (1 << PCIE0)
#define serial_interrupt_pin (1 << PCINT7)
#define interrupt_vector PCINT0_vect
#define mask_pin_change_register PCMSK0
#define mask_general_interrupt_register GIMSK

#define ROWS 6
#define COLS 2

// Map the pins tuples
const charliePinType led[COLS][ROWS] = {
	{ {B,A}, {C,A}, {A,B}, {C,B}, {A,C}, {B,C} },
	{ {E,D}, {F,D}, {D,E}, {F,E}, {D,F}, {E,F} }
};

uint8_t led_is_on[COLS][ROWS]; 

typedef struct {
	uint8_t col_zero;
	uint8_t col_one; 
} recipeType;

const recipeType cocktails[] = {
	// Bits from right to left: 
	{0x00, 0x00}, // 'a': clear
	{0x0B, 0x04}, // 'b': b1011, b100 - "Cuba Libre": {"Rum" : 1.7, "Cola" : 0.816, "Fresh lime juice" : 0.34},
	{0x17, 0x08}, // 'c': b10111, b1000 - "Margarita"": {"Lime" : 3, "Tequila" : 0.7, "Triple Sec" : 0.25},
	{0x03, 0x04}  // 'd': b11, b100 - "Dry Martini": {"Gin" : 2.02, "Dry vermouth": 0.34} 
};

int flag = 0;

void matrix_display(uint8_t delay) {
	//
	// Display this while not handling interrupts
	//
	int i, j;
	for (i=0; i<COLS; ++i) {
		for (j=0; j<ROWS; ++j) {
			flash(led[i][j], 100);
		}
	}
}


ISR(interrupt_vector) {
	//
	// pin change interrupt handler
	//
	static char chr; 		// recipeType is passed by character
	uint8_t row_mask = 0x01;// binary 000001 = rightmost bit
	int row_index = 0;		// 
	//
	// Get selected recipte form issues
	//
	get_char(&serial_in_pins, serial_pin_in, &chr);
	//
	// determine which rows to light: 0 is off, 1 is on
	//
	int recipe_num = chr - 'a';
	recipeType cocktail = cocktails[recipe_num];// get respective index in array
												// TODO: verify value is out of bounds
	//
	// iterate through bit mask shifting 1 to the left.
	// 6 LEDs are represented by 6 bits; max value is b111111 = 0x3f
	//
	for (; row_mask<0x3f; row_mask <<= 1) { 
		//
		// Condition is fulfilled if result isn't 0, which happens
		// when the evaluated bit is 1
		//
		led_is_on[0][row_index] = (cocktail.col_zero & row_mask) ? 1 : 0;
		led_is_on[1][row_index] = (cocktail.col_one & row_mask) ? 1 : 0;
		++row_index;
	}
	//
 	// clear interrupt flag for PCINT0 to ignore duplicate interrupts
	// see Visovian's comment at 
	// http://www.avrfreaks.net/index.php?name=PNphpBB2&file=printview&t=98538&start=0
	//
	GIFR = (1 << PCIF0);
}


int main(void) {
	//
	// main
	//
	int i, j;
	
	// All LEDs should be off
	for (i=0; i<COLS; ++i) {
		for (j=0; j<ROWS; ++j) {
			led_is_on[i][j] = 0;
		}
	}
	//
	// set clock divider to /1
	//
	CLKPR = (1 << CLKPCE);
	CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
	//
	// set up pin change interrupt on input pin
	//
	set(mask_general_interrupt_register, serial_interrupt);
	set(mask_pin_change_register, serial_interrupt_pin);
	sei();	// enable interrupts
	//
	// initialize output pins
	//
	set(serial_out_port, serial_pin_out);
	output(serial_out_direction, serial_pin_out);
	//
	// main loop
	//
	while (1) {
		//get_char(&serial_in_pins, serial_pin_in, &chr);
		//put_char(&serial_in_port, serial_pin_out, chr);
		
		for (i=0; i<COLS; ++i) {
			for (j=0; j<ROWS; ++j) {
				if (led_is_on[i][j]) {
					flash(led[i][j], 1);
				}
			}
		} // end for
	} // end while
}